import os
from .route_holder import RouteHolder
from .response_handler import ResponseHandler

def all_subclasses(cls):
  return set(cls.__subclasses__()).union(
    [s for c in cls.__subclasses__() for s in all_subclasses(c)]
  )

class RequestHandler:
  def __init__(self, context):
    self.context = context
    self.router = RouteHolder()
    self.respondent = ResponseHandler()

  def _find_best_route(self, event):
    path = event['pathParameters'].get('proxy', '')
    http_method = event['httpMethod']
    return self.router.find_best_route(path, http_method)

  def _format_response(self, result):
    # Move this to the respondent
    return result

  def handle(self, event):
    best_handler = self._find_best_route(event)
    if best_handler:
      result = best_handler(event, respondent=self.respondent)
      if result:
        self.respondent.set_body(result)
    else:
      self.respondent.set_status(404)

    return self.respondent.response


import setuptools

with open("README.md", "r") as fh:
  long_description = fh.read()

setuptools.setup(
  name="py-lambda-base", # Replace with your own username
  version="0.0.6-dev11",
  author="Michael Madison",
  author_email="cadetstar@hotmail.com",
  description="Pseudo-application for AWS Lambda",
  long_description=long_description,
  long_description_content_type="text/markdown",
  url="https://github.com/pypa/sampleproject",
  packages=setuptools.find_packages(),
  install_requires=[
    'deepmerge==0.1.0',
    'dpath==2.0.1',
    'json-api-doc==0.11.0'
  ],
  classifiers=[
    "Programming Language :: Python :: 3",
    "License :: OSI Approved :: MIT License",
    "Operating System :: OS Independent",
  ],
  python_requires='>=3.6',
)
